import os

# Path to database file. You need to touch it by yourself
DB_PATH = os.path.join(os.path.expanduser("~"), ".seasondog", "database")

# Command to execute for playing videofile. 
PLAYER = "mplayer -lavdopts threads=4 -fs {player_args} \"{videofile}\""

# Extensions for lookuping subs
# from wiki
SUB_EXTENSIONS = ["srt", "ass", "aqt", "gsub", "sub", "ttxt", "pjs", "psb", "rt", "stl", "ssa", "usf", "idx",]

# Extensions for lookuping videofiles
# from .dir_colors
MOVIE_EXTENSIONS = ["mov","mpg","mpeg","m2v","mkv","webm","ogm","mp4","m4v","mp4v","vob","qt","nuv","wmv","asf","rm","rmvb","flc","avi","fli","flv","gl","dl","xcf","xwd","yuv","cgm","emf",]

COLORS = {
    "blue": "\033[94m",
    "magenta": "\033[95m",
    "green": "\033[92m",
    "yellow": "\033[93m",
    "grey": "\033[90m",
    "red": "\033[91m",

    "bold": "\033[1m",

    "endc": "\033[0m",
}

COLORS.update({
    "c_title": COLORS["grey"],
    "c_ep_number": COLORS["blue"] + COLORS["bold"],
    "c_path": COLORS["blue"],
    "c_args": COLORS["blue"],
    "c_control": COLORS["grey"],
})



import os
import re
from seasondog.config import MOVIE_EXTENSIONS, SUB_EXTENSIONS

def leading_zero(episode, count):
    return ''.join(['0' for i in range(count - len(str(episode)))]) + str(episode)

def re_matcher(pattern):
    return lambda x, n: re.match(pattern, x, re.I)

def simple_matcher(episode, extensions=[], zero=2):
    return re_matcher(r"^.*{}.*({})$".format(leading_zero(episode, zero), "|".join(extensions)))

def prefix_matcher(prefix, episode, extensions=[], zero=2):
    return re_matcher(r"^.*{}{}.*({})$".format(prefix, leading_zero(episode, zero), "|".join(extensions)))

def prefix_suffix_matcher(prefix, suffix, episode, extensions=[], zero=2):
    return re_matcher(r"^.*{}{}{}.*({})$".format(prefix, leading_zero(episode, zero), suffix, "|".join(extensions)))

def nth_matcher(episode):
    return lambda x, n: n == episode

def match(directory, matchers, limit=1):
    for path, dirs, files in os.walk(directory):
        files.sort()

        for matcher in matchers:
            result = []
            i = 0
            for x in files:
                i += 1
                if matcher(x, i):
                    result.append(os.path.join(path, x))

            if result and (limit == -1 or len(result) == limit):
                return result

def match_episode(directory, episode, limit=1):
    prefixes = ["episode", "ep", "e", "x"]
    suffixes = [" ", ""]
    matchers = []
    for p in prefixes:
        for s in suffixes:
            for i in range(1, 3):
                matchers.append(prefix_suffix_matcher(p, s, episode, MOVIE_EXTENSIONS, i))

    matchers.append(simple_matcher(episode, MOVIE_EXTENSIONS))
    matchers.append(nth_matcher(episode))

    return match(directory, matchers, limit)

def match_subs(directory, episode, limit=1):
    prefixes = ["episode", "ep", "e", "x"]
    suffixes = [" ", ""]
    matchers = []
    for p in prefixes:
        for s in suffixes:
            for i in range(1, 3):
                matchers.append(prefix_suffix_matcher(p, s, episode, SUB_EXTENSIONS, i))

    matchers.append(simple_matcher(episode, SUB_EXTENSIONS))
    matchers.append(nth_matcher(episode))

    return match(directory, matchers, limit)

def match_file(directory, episode, limit=1):
    return match(
            directory, [
            simple_matcher(episode),
            nth_matcher(episode),
            ],
            limit)



import os
import copy
from seasondog import info

NAME = "plain"
VERSION = 0.1

DB = 1
TRANSACTION = 2
PATH = 3
PLAYER_ARGS = 4
EPISODE = 5 

STRLEN = 128

def increment(a):
    return a + 1

def text(s, ln):
    if len(s) <= ln:
        return s + "".join([chr(0) for i in range(ln - len(s))])
    else:
        raise RuntimeException("Str value out of STRLEN!")

def db_struct(path, db={}, transaction={}):
    return {DB: db,
            TRANSACTION: transaction,
            PATH: path,
            }

def db_value(directory, episode, args):
    return text("{}:{}:{}".format(directory, episode, args), STRLEN)

def init(file):
    with open(file, 'w') as f:
        f.write("{}:{},{}:{}\n".format(info.NAME, info.VERSION, NAME, VERSION))
    
    return db_struct(file)

def load(file):
    db = db_struct(file)

    dir = os.path.join(*os.path.split(file)[:-1])
    if not os.path.exists(dir):
        os.makedirs(dir)

    if not os.path.exists(file):
        open(file, 'w').close()

    with open(file) as f:
        f.readline() # @TODO: header check
        line = f.readline()
        while line:
            directory, episode, player_args = line.strip().split(":")
            player_args = player_args.replace(chr(0), "")

            db[DB][directory] = {
                    EPISODE: int(episode),
                    PLAYER_ARGS: player_args,
                    }

            line = f.readline()

    return db

def save(db):
    init(db[PATH])
    data = db[DB].copy()
    for k, v in db[TRANSACTION].items():
        data[k] = v

    with open(db[PATH], 'a') as f:
        for directory, option in data.items():
            f.write(db_value(directory, option[EPISODE], option[PLAYER_ARGS]) + "\n")

def commit(db, file):
    map = {}
    
    index = 1
    with open(file) as f:
        f.readline() # @TODO: header check
        line = f.readline()
        while line:
            directory, episode = line.strip().split(":")
            if directory in db[TRANSACTION]:
                map[index] = [directory, db[TRANSACTION][directory]]
                db[DB][directory] = db[TRANSACTION][directory]
                del db[TRANSACTION][directory]
            index += 1
            line = f.readline()
    
    with open(file, 'a') as f:
        f.seek(0)
        for line, [directory, episode] in map.items():
            f.write(db_value(directory, episode) + "\n")


def get(db, directory):
    return db[TRANSACTION].get(directory, db[DB].get(directory))

def set(db, directory, data):
    db[TRANSACTION][directory] = data
    return db

def update(db, directory, fn, *args, **kwargs):
    return set(db, directory, fn(get(db, directory), *args, **kwargs))



import os
import re
from seasondog import matcher
from seasondog import database as database
from seasondog import runtime as r
from seasondog import config

def player_args(runtime, data, string):
    for call in re.finditer(r"(\@(?P<fn>[a-zA-Z_]+)\((?P<args>[^@]*)\))", string):
        match = call.group(1)
        args = call.group("args").split(" ")
        fn = call.group("fn")

        if fn == "subs":
            limit = int(args[1]) if len(args) > 1 else 1
            delim = args[2] if len(args) > 2 else " "

            files = matcher.match_subs(args[0], data[database.EPISODE], -1)
            if files:
                if limit != -1:
                    files = files[:limit]
                string = string.replace(match, "\"{}\"".format(delim.join(files)))
            else:
                string = string.replace(match, "")

        elif fn == "files":
            limit = int(args[1]) if len(args) > 1 else 1
            delim = args[2] if len(args) > 2 else " "

            files = matcher.match_file(args[0], data[database.EPISODE], -1)
            if files:
                if limit != -1:
                    files = files[:limit]
                string = string.replace(match, "\"{}\"".format(delim.join(files)))
            else:
                string = string.replace(match, "")

    return string
        
def watch(runtime, data):
    episode = data[database.EPISODE]

    file = matcher.match_episode(runtime[r.PATH], episode)[0]
    if file:
        if not runtime[r.PLAYER_ARGS_OVERRIDE]:
            args = player_args(runtime, data, data[database.PLAYER_ARGS])
        else:
                args = player_args(runtime, data, runtime[r.PLAYER_ARGS_OVERRIDE])

        print(r.format("{c_title}Episode {c_ep_number}{bold}#{episode}{endc}\n        {c_path}{path}\n        {c_args}{args}\n{endc}{c_control}[[enter] - watch, [^c] - break]{endc}", 
            episode=episode, 
            path=os.path.split(file)[-1],
            args=args,
            ))
        try:
            input()
        except KeyboardInterrupt:
            return
        
        os.system(config.PLAYER.format(player_args=args, videofile=file))
    else:
        print("Episode #{}\n        No episodes matched!".format(episode))




from optparse import OptionParser
from seasondog import config

PATH = 1
DB_PATH = 2
PLAYER_ARGS_OVERRIDE = 3

def runtime_struct(path, opt):
    return {PATH: path,
            DB_PATH: config.DB_PATH,
            PLAYER_ARGS_OVERRIDE: opt.player_args,
            }

def format(s, *args, **kwargs):
    kwargs.update(config.COLORS)
    return s.format(*args, **kwargs)



#!/usr/bin/env python

from distutils.core import setup
from seasondog import info

setup(name='seasondog',
      version=info.VERSION,
      description='Tool for tracking you progress in watching series and playing you the next episode',
      author='Vasiliy Horbachenko',
      author_email='shadow.prince@ya.ru',
      url=info.URL,
      scripts=['seasondog/sdog'],
      packages=['seasondog', ],
     )
